/* .COPYRIGHT:                                                                */
/*    Licensed Material - Program-Property of IBM                             */
/*    (C) Copyright IBM Corp. 1999, 2001 - All Rights Reserved                */
/*                                                                            */
/* .DISCLAIMER:                                                               */
/*   The following [enclosed] code is sample code created by IBM              */
/*   Corporation.  This sample code is provided to you solely for use in      */
/*   connection with your internal evaluation of the Service Management       */
/*   Framework(TM) Toolkit (Program) during the term of Early Release license */
/*   for the Program, and for no other purpose.  The code is provided 'AS IS',*/
/*   without warranty of any kind.  IBM shall not be liable for any damages   */
/*   arising out of your use of the sample code, even if they have been       */
/*   advised of the possibility of such damages.                              */

package com.ibm.osg.sample.pizza;

import org.osgi.framework.*;
import org.osgi.service.http.*;
import org.osgi.util.tracker.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.net.URL;
import java.security.AccessController;
import java.security.PrivilegedAction;

/**
 * This class is the main class for the Pizza sample. It is the
 * BundleActivator and is responsible for starting and stopping the
 * bundle. It also manages the use of the HttpService and registers
 * the Pizza servlet.
 */
public class PizzaBundle implements BundleActivator, ServiceTrackerCustomizer
{
    /** BundleContext for this bundle */
    protected BundleContext context;

    /** Log Service wrapper object */
    protected LogTracker log;

    /** Http Service tracker object */
    protected ServiceTracker tracker;

    /** HttpContext for HTTP registrations */
    protected HttpContext httpContext;

    /** Servlet URI */
    protected String servletURI = "/pizza";

    /** Servlet images URI suffix */
    protected String imagesURI = "/images";

    /**
     * Must have a public default constructor!
     * It doesn't have to do anything but the BundleActivator
     * class must be able to be instantiated by Class.newInstance().
     *
     * This same instance of this object that is used to start
     * the bundle will be used to stop the bundle.
     */
    public PizzaBundle()
    {
    }

    /**
     * Part of the BundleActivator interface. This method starts
     * the bundle and must quickly return. If the bundle requires
     * its own threads. they can be created here. If the bundle
     * registers any Services, they can be registered here.
     *
     * @param   context BundleContext object for this bundle
     */
    public void start(BundleContext context) throws Exception
    {
        this.context = context;

        httpContext = new HttpContext()
        {
            public boolean handleSecurity(HttpServletRequest request,
                                          HttpServletResponse response)
            {
                return true;
            }

            public URL getResource(final String name)
            {
                return (URL) AccessController.doPrivileged(new PrivilegedAction()
                {
                    public Object run()
                    {
                        String resource = name;

                        if (resource.charAt(0) != '/')
                        {
                            resource = "/".concat(resource);
                        }

                        return getClass().getResource(resource);
                    }
                });
            }

            public String getMimeType(String name)
            {
                return null;
            }
        };

        log = new LogTracker(context, System.err);

        tracker = new ServiceTracker(context, HttpService.class.getName(), this);
        tracker.open();
    }

    /**
     * Part of the BundleActivator interface. This method stops
     * the bundle and must quickly return. If the bundle registered any
     * Services they should be unregistered here.
     *
     * @param   context BundleContext object for this bundle
     */
    public void stop(BundleContext context) throws Exception
    {
        tracker.close();
        log.close();
    }

    /**
     * An Http Service is being added to the ServiceTracker object.
     *
     * @param reference Reference to service being added to the <tt>ServiceTracker</tt> object.
     * @return The service object to be tracked for the
     * <tt>ServiceReference</tt> object or <tt>null</tt> if the <tt>ServiceReference</tt> object should not
     * be tracked.
     */
    public Object addingService(ServiceReference reference)
    {
        HttpService http = (HttpService)context.getService(reference);

        if (http != null)
        {
            try
            {
                http.registerServlet(servletURI, new Pizza(), null, httpContext);
                http.registerResources(servletURI+imagesURI, imagesURI, httpContext);

                log.log(log.LOG_INFO, "Pizza Servlet registered");
            }
            catch (ServletException e)
            {
                Throwable n = e.getRootCause();
                if (n == null)
                {
                    n = e;
                }

                log.log(log.LOG_ERROR, "registerServlet ServletException", n);

                context.ungetService(reference);
                http = null;
            }
            catch (NamespaceException e)
            {
                Throwable n = e.getException();
                if (n == null)
                {
                    n = e;
                }

                log.log(log.LOG_ERROR, "registerServlet NamespaceException", n);

                context.ungetService(reference);
                http = null;
            }
        }

        return http;
    }

    /**
     * An Http Service tracked by the ServiceTracker object has been modified.
     *
     * @param reference Reference to service that has been modified.
     * @param service The service object for the modified service.
     */
    public void modifiedService(ServiceReference reference, Object service)
    {
    }

    /**
     * An Http Service tracked by the ServiceTracker object has been removed.
     *
     * <p>This method is called after a service is no longer being tracked
     * by the <tt>ServiceTracker</tt> object.
     *
     * @param reference Reference to service that has been removed.
     * @param service The service object for the removed service.
     */
    public void removedService(ServiceReference reference, Object service)
    {
        HttpService http = (HttpService) service;

        http.unregister(servletURI);
        http.unregister(servletURI+imagesURI);

        context.ungetService(reference);
        log.log(log.LOG_INFO, "Pizza Servlet unregistered");
    }
}
