package com.ibm.osg.sample.simple;

/*
 * (c) Copyright IBM Corp. 2002.  All Rights Reserved
 */

import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.osgi.service.log.LogService;

/**
 * A BundleActivator that sends a log message to
 * the standard OSGi LogService when started and stopped.
 * <p>
 * If the LogService is not available then the message
 * is written to <code>System.out</code> instead.
 * <p>
 * <pre>(c) Copyright IBM Corp 2002</pre>
 *
 * @author OTI
 * @version 2.0
 */
public class SimpleBundleActivator extends Object implements BundleActivator {

	/** The context under which this bundle executes. */
	private BundleContext bc;

	/** The name of the LogService */
	static final String logServiceName = LogService.class.getName();

	/**
	 * Default constructor.
	 */
	public SimpleBundleActivator() {
		super();
	}

	/**
	 * Starts the bundle.
	 *
	 * @param bc the BundleContext under which this bundle will execute
	 */
	public void start(BundleContext bc) throws Exception {
		this.bc = bc;

		String message = getBundleName() + ": Hi";

		if (!log(LogService.LOG_INFO, message, null)) {
			print(LogService.LOG_INFO, message, null);
		}
	}

	/**
	 * Stops the bundle.
	 *
	 * @param bc the BundleContext under which this bundle executes
	 */
	public void stop(BundleContext bc) throws Exception {
		String message = getBundleName() + ": Bye";

		if (!log(LogService.LOG_INFO, message, null)) {
			print(LogService.LOG_INFO, message, null);
		}

		this.bc = null;
	}

	/**
	 * Records an event in the LogService.
	 *
	 * @param severity the integer severity code
	 * @param message a java.lang.String containing the message to be recorded in the log
	 * @param e the java.lang.Throwable which triggered the event being recorded
	 * 
	 * @return <code>true</code> if the message could be logged, <code>false</code> otherwise
	 */
	protected boolean log(int severity, String message, Throwable e) {
	
		// Get the reference to the LogService
		ServiceReference logRef = bc.getServiceReference(logServiceName);
		if (logRef != null) {
			// Get the LogService object
			LogService log = (LogService) bc.getService(logRef);
			if (log != null) {
				try {
					// Write to the LogService
					log.log(severity, message, e);
					return true;
				} finally {
					// Release the reference to the LogService
					bc.ungetService(logRef);
				}
			}
		}
		return false;
	}

	/**
	 * Prints the message to System.out or System.err if the severity is an Error.
	 *
	 * @param severity the integer severity code
	 * @param message a java.lang.String containing the message to be recorded in the log
	 * @param e the java.lang.Throwable which triggered the event being recorded
	 */
	protected void print(int severity, String message, Throwable e) {

		switch (severity) {
			case LogService.LOG_ERROR :
				System.err.println("Error: " + message);
				break;
			case LogService.LOG_WARNING :
				System.out.println("Warning: " + message);
				break;
			case LogService.LOG_INFO :
				System.out.println(message);
				break;
			default :
				// don't print debug messages
		}

		if (e != null) {
			e.printStackTrace(System.err);
		}
	}

	/**
	 * Returns the name of this bundle as defined in the bundle's
	 * Manifest file or <code>null</code> if the bundle name is not
	 * defined or the bundle is not started or does not have the
	 * AdminPermission when run in a secure OSGi framework.
	 */
	public String getBundleName() {
		try {
			return (String) bc.getBundle().getHeaders().get("Bundle-Name");
		} catch (NullPointerException e) {
		} catch (IllegalStateException e) {
			// The bundle has been stopped
		} catch (SecurityException e) {
			// The bundle does not have the permission to read the headers
		}
		return null;
	}
}