/* .COPYRIGHT:                                                                */
/*    Licensed Material - Program-Property of IBM                             */
/*    (C) Copyright IBM Corp. 1999, 2001 - All Rights Reserved                */
/*                                                                            */
/* .DISCLAIMER:                                                               */
/*   The following [enclosed] code is sample code created by IBM              */
/*   Corporation.  This sample code is provided to you solely for use in      */
/*   connection with your internal evaluation of the Service Management       */
/*   Framework(TM) Toolkit (Program) during the term of Early Release license */
/*   for the Program, and for no other purpose.  The code is provided 'AS IS',*/
/*   without warranty of any kind.  IBM shall not be liable for any damages   */
/*   arising out of your use of the sample code, even if they have been       */
/*   advised of the possibility of such damages.                              */

package com.ibm.osg.sample.simplepermissionpolicy;

import java.io.PrintStream;
import org.osgi.framework.*;
import org.osgi.service.log.*;
import org.osgi.util.tracker.*;

/**
 * LogTracker class. This class encapsulates the LogService
 * and handles all issues such as the service coming and going.
 *
 * @author BJ Hargrave (hargrave@us.ibm.com)
 * @version 2.0
 */

public class LogTracker extends ServiceTracker implements LogService
{
    /** LogService interface class name */
    protected final static String clazz = "org.osgi.service.log.LogService";

    /** PrintStream to use if LogService is unavailable */
    protected PrintStream out;


    /**
     * Create new LogTracker.
     *
     * @param context BundleContext of parent bundle.
     * @param out Default PrintStream to use if LogService is unavailable.
     */
    public LogTracker(BundleContext context, PrintStream out)
    {
        super(context, clazz, null);
        this.out = out;

        open();
    }

    /*
     * ----------------------------------------------------------------------
     *      LogService Interface implementation
     * ----------------------------------------------------------------------
     */

    public void log(int level, String message)
    {
        log(null, level, message, null);
    }

    public void log(int level, String message, Throwable exception)
    {
        log(null, level, message, exception);
    }

    public void log(ServiceReference reference, int level, String message)
    {
        log(reference, level, message, null);
    }

    public synchronized void log(ServiceReference reference, int level, String message,
                                 Throwable exception)
    {
        ServiceReference[] references = getServiceReferences();

        if (references != null)
        {
            int size = references.length;

            for (int i = 0; i < size; i++)
            {
                LogService service = (LogService)getService(references[i]);
                if (service != null)
                {
                    try
                    {
                        service.log(reference, level, message, exception);
                    }
                    catch (Exception e)
                    {
                    }
                }
            }

            return;
        }

        noLogService(level, message, exception, reference);
    }

    /**
     * The LogService is not available so we write the message to a PrintStream.
     *
     * @param level Logging level
     * @param message Log message.
     * @param throwable Log exception or null if none.
     * @param reference ServiceReference associated with message or null if none.
     */
    protected void noLogService(int level, String message, Throwable throwable, ServiceReference reference)
    {
        if (out != null)
        {
            synchronized (out)
            {
                switch (level)
                {
                    case LOG_DEBUG:
                    {
                        out.print("Debug: ");

                        break;
                    }
                    case LOG_INFO:
                    {
                        out.print("Info: ");

                        break;
                    }
                    case LOG_WARNING:
                    {
                        out.print("Warning: ");

                        break;
                    }
                    case LOG_ERROR:
                    {
                        out.print("Error: ");

                        break;
                    }
                    default:
                    {
                        out.print("Unknown Log level[");
                        out.print(level);
                        out.print("]: ");

                        break;
                    }
                }

                out.println(message);

                if (reference != null)
                {
                    out.println(reference);
                }

                if (throwable != null)
                {
                    throwable.printStackTrace(out);
                }
            }
        }
    }
}
